/*
 * file CqFieldDefinition.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqFieldDefinition
 *
 * (C) Copyright IBM Corporation 2004, 2015.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;
import java.util.Map;

import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpProperty;
import com.ibm.rational.wvcm.stp.StpResource;
import com.ibm.rational.wvcm.stp.StpProperty.Type;
import com.ibm.rational.wvcm.stp.cq.CqQuery.Filter.Operation;
import com.ibm.rational.wvcm.stpex.StpExBase;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;


/**
 * A client proxy for a field definition metadata object.
 * 
 * <p>
 * This object defines the characteristics of a field of a record type in a
 * database schema. Since field definitions are schema objects, they cannot be
 * created, modified, or deleted by the client. Any client may assume that a
 * field definition will remain unchanged for the duration of an active session
 * and therefore, the client need only fetch field definitions from the server
 * once per session. Field definitions are usually fetched when the CqRecordType
 * resource of which they are a part is fetched.
 * <p>
 * An array of CqFieldDefintions is used to define a query field path, used in
 * both {@link CqQuery.DisplayField} and {@link CqQuery.FilterLeaf}
 * specifications.
 * <p>
 * CqFieldDefinitions are also used in a {@link CqUserDb.SearchFilter} to limit
 * a full-text search to values in specific fields.
 * <p>
 * The user-friendly specification for the location of a field definition has the form
 * <pre>
 *  <b>cq.field:</b><i>&lt;record-type&gt;</i>/<i>&lt;field-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 */
public interface CqFieldDefinition extends CqUserDbMember,
                CqUserDb.SearchKey<List<String>>
{
    /**
     * An enumeration of the possible restrictions on assigning a value to a
     * field.
     */
    public enum Requiredness implements StpExEnumeration
    {
        /** field is READ_ONLY; its value cannot be changed. */
        READ_ONLY,

        /** a field value is MANDATORY; a non-null value must be provided. */
        MANDATORY,

        /**
         * a field value is OPTIONAL; a non-null value may be provider but is
         * not required
         */
        OPTIONAL,

        /** a field value is determined by calling the associated hook */
        USE_HOOK;
    }

    /**
     * The data type of the value of the field defined by this field definition.
     */
    public static final PropertyName<CqFieldValue.ValueType> FIELD_TYPE =
        new PropertyName<CqFieldValue.ValueType>(PROPERTY_NAMESPACE,
                                                 "field-type");

    /**
     * Returns the value of the {@link #FIELD_TYPE} property as defined by this
     * proxy.
     * 
     * @return The field value type as a CqFieldValue.ValueType enumerator. Will
     *         never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #FIELD_TYPE} property.
     */
    public CqFieldValue.ValueType getFieldType() throws WvcmException;

    /** A short phrase describing the purpose and function of this field. */
    PropertyName<String> HELP_TEXT =
        new PropertyName<String>(PROPERTY_NAMESPACE, "help-text");

    /**
     * Returns the value of the {@link #HELP_TEXT HELP_TEXT} property as defined
     * by this proxy.
     * 
     * @return A string containing a short description of the role of this field
     *         in the schema.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #HELP_TEXT HELP_TEXT} property.
     */
    public String getHelpText() throws WvcmException;

    /** Record type of the record referenced by this field */
    PropertyName<CqRecord> REFERENCED_RECORD_TYPE =
        new PropertyName<CqRecord>(PROPERTY_NAMESPACE, "referenced-record-type");

    /**
     * Returns the value of the {@link #REFERENCED_RECORD_TYPE
     * REFERENCED_RECORD_TYPE} property as defined by this proxy.
     * 
     * @return A CqRecordType proxy representing the type of record that
     *         that is referenced by this field; <b>null</b> if this field is 
     *         not type {@link CqFieldValue.ValueType#RESOURCE} or
     *         {@link CqFieldValue.ValueType#RESOURCE_LIST}
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #REFERENCED_RECORD_TYPE REFERENCED_RECORD_TYPE}
     *             property.
     */
    public CqRecordType getReferencedRecordType() throws WvcmException;

    /**
     * A CqRecordType proxy for the record-type resource of which this resource
     * defines a field.
     */
    PropertyName<CqRecordType> RECORD_TYPE =
        new PropertyName<CqRecordType>(PROPERTY_NAMESPACE, "record-type");

    /**
     * Returns the value of the {@link #RECORD_TYPE RECORD_TYPE} property as
     * defined by this proxy.
     * 
     * @return The CqRecordType proxy for the record-type that contains this
     *         field definition.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #RECORD_TYPE RECORD_TYPE} property.
     */
    public CqRecordType getRecordType() throws WvcmException;

    /**
     * The schema-defined behavior (read-only, mandatory, or optional) of this
     * field for each state name in the record type's state table. (The
     * schema-defined behavior for a field of a stateless record type is
     * associated with the state name "*".) Also includes the default,
     * system-imposed requiredness for the field as the value associated with
     * the state name "".
     */
    PropertyName<Map<String, Requiredness>> REQUIREDNESS_BY_STATE =
        new PropertyName<Map<String, Requiredness>>(PROPERTY_NAMESPACE,
                                                    "requiredness-by-state");

    /**
     * Returns the value of the {@link #REQUIREDNESS_BY_STATE
     * REQUIREDNESS_BY_STATE} property as defined by this proxy.
     * 
     * @return A Map from String to Requiredness objects specifying for each
     *         state of the record the Requiredness of this field. For stateless
     *         record types, which have no states, the "*" key is mapped to the
     *         schema-defined requiredness for the field. In all cases, the
     *         empty key ("") is mapped to the default (system-imposed)
     *         requiredness for the field.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #REQUIREDNESS_BY_STATE REQUIREDNESS_BY_STATE}
     *             property.
     */
    public Map<String, Requiredness> getRequirednessByState()
        throws WvcmException;

    /**
     * A list of possible values for fields of this type.
     * <p>
     * In general, the content of a field's choice list is quite volatile and
     * can depend on the state of the record it is in, the value of other fields
     * of that record or related records, the current user's permissions, and
     * other dynamic aspects of the database environment known only to the
     * schema. So, in general, the choice list must be obtained in the context
     * in which it is to be used at the time it is to be used. In this
     * interface, that very dynamic value is expressed as the value of the
     * {@link CqFieldValue#CHOICE_LIST CHOICE_LIST} meta-property of the
     * CqFieldValue property representing the field of the record that is to be
     * set with a value from the choice list.
     * <p>
     * {@link CqFieldValue#CHOICE_LIST} always returns the choice list for the
     * field based on all factors that could influence its content. The values
     * returned by this property are based only on the current state of the
     * database, but not on the state of any specific record. If the schema is
     * designed such that the choice list for this field does not depend on the
     * value of other fields in the record, then the list returned by this
     * property will be the same as that returned by
     * {@link CqFieldValue#CHOICE_LIST}.
     * <p>
     * Currently, there is no way for a client to interrogate a schema to
     * discover the dynamics of its choice list computations. When presenting
     * choices to a user for the purpose of actually setting a field value into
     * a specific record, {@link CqFieldValue#CHOICE_LIST} should always be used.
     * In contexts where no specific record is indicated (as in defining a query,
     * for example) the value of this property would be most appropriate.
     * <p>
     * If this property is empty, it neither implies that all values are
     * permitted nor that no values are permitted; it just means that the schema
     * designer has not provided any hints about the values permitted in the
     * field.
     * <p>
     * Choice lists can be voluminous and, so, should be read from the server
     * only when needed. Yet, as explained above, they can also be quite
     * volatile and may easily change during the course of a session and not
     * necessarily in direct response to anything the client is doing.
     * 
     * @see CqFieldValue#CHOICE_LIST
     */
    PropertyName<List<String>> CHOICE_LIST =
        new PropertyName<List<String>>(PROPERTY_NAMESPACE, "choice-list");

    /**
     * Returns the value of the {@link #CHOICE_LIST CHOICE_LIST}
     * property as defined by this proxy.
     * 
     * @return A List of the values defined statically by the database schema as
     *         valid values for fields of this description.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CHOICE_LIST CHOICE_LIST} property.
     */
    public List<String> getChoiceList() throws WvcmException;

    /**
     * Whether or not values not in the field's choice list are permitted as
     * legal field values.
     * <p>
     * The schema designer specifies whether the legal values for a given field
     * are restricted to the contents of its choice list and that specification
     * is captured by this property. If there is a restriction, specifying a
     * value not in the choice list causes a validation error. If there is no
     * restriction, values not in the choice list may be used. (Note that any
     * values must still be validated by the schema at the time the record is
     * delivered to the database.)
     */
    PropertyName<Boolean> HAS_OPEN_CHOICE_LIST =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-open-choice-list");

    /**
     * Returns the value of the {@link #HAS_OPEN_CHOICE_LIST
     * HAS_OPEN_CHOICE_LIST} property as defined by this proxy.
     * Requires SQL-editor permission.
     * 
     * @return <b>True</b> if legal values for this field are <i>not</i>
     *         constrained to those that appear in its choice list.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #HAS_OPEN_CHOICE_LIST HAS_OPEN_CHOICE_LIST} property.
     * 
     */
    public boolean getHasOpenChoiceList() throws WvcmException;

    /**
     * Whether or not the content of the field's choice list are computed with
     * the aid of a hook-script.
     * 
     * Requires SQL-editor permission
     */
    PropertyName<Boolean> HAS_SCRIPTED_CHOICE_LIST =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "has-scripted-choice-list");

    /**
     * Returns the value of the {@link #HAS_SCRIPTED_CHOICE_LIST
     * HAS_SCRIPTED_CHOICE_LIST} property as defined by this proxy.
     * 
     * @return <b>True</b> the content of the field's choice list are computed
     *         with the aid of a hook-script; <b>False</b> if not.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #HAS_OPEN_CHOICE_LIST HAS_OPEN_CHOICE_LIST} property.
     * 
     * Requires SQL-editor permission
     */
    public boolean getHasScriptedChoiceList() throws WvcmException;

    /**
     * Indicates that the content of the choice list for this field is liable to
     * be recalculated each time it is accessed. Such choice lists probably
     * should not be cached on the client.
     * 
     * Requires SQL-editor permission
     */
    PropertyName<Boolean> CALCULATES_CHOICE_LIST_EACH_ACCESS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "calculates-choice-list-each-access");

    /**
     * Returns the value of the {@link #CALCULATES_CHOICE_LIST_EACH_ACCESS
     * CALCULATES_CHOICE_LIST_EACH_ACCESS} property as defined by this proxy.
     * 
     * @return <b>True</b> if the choice list for this field is liable to be
     *         recalculated each time it is referenced.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CALCULATES_CHOICE_LIST_EACH_ACCESS CALCULATES_CHOICE_LIST_EACH_ACCESS}
     *             property.
     * 
     * Requires SQL-editor permission
     */
    public boolean getCalculatesChoiceListEachAccess() throws WvcmException;

    /**
     * Identifies the dynamic choice list used by this field as its choice list
     * 
     * Requires SQL-editor permission
     */
    PropertyName<CqDynamicChoiceList> DYNAMIC_CHOICE_LIST =
        new PropertyName<CqDynamicChoiceList>(PROPERTY_NAMESPACE,
                                              "dynamic-choice-list");

    /**
     * Returns the value of the {@link #DYNAMIC_CHOICE_LIST DYNAMIC_CHOICE_LIST}
     * property as defined by this proxy.
     * 
     * @return A CqDynamcChoiceList proxy for the dynamic choice list used by
     *         this field as its choice list; <b>null</b> if this field does
     *         not use a dynamic choice list as its choice list.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DYNAMIC_CHOICE_LIST DYNAMIC_CHOICE_LIST} property.
     * 
     * Requires SQL-editor permission
     */
    public CqDynamicChoiceList getDynamicChoiceList() throws WvcmException;

    /**
     * The maximum length of a string permitted as this field's value.
     * 
     * This property requires the CQ SQL Editor permission.
     */
    PropertyName<Long> MAXIMUM_STRING_LENGTH =
        new PropertyName<Long>(PROPERTY_NAMESPACE, "maximum-string-length");

    /**
     * Returns the value of the {@link #MAXIMUM_STRING_LENGTH
     * MAXIMUM_STRING_LENGTH} property as defined by this proxy.
     * 
     * @return The maximum string length permitted for the value of this field.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #MAXIMUM_STRING_LENGTH MAXIMUM_STRING_LENGTH}
     *             property.
     * 
     * Requires SQL-editor permission
     */
    public long getMaximumStringLength() throws WvcmException;

    /** Whether or not this field can be a security context field */
    PropertyName<Boolean> CAN_BE_SECURITY_CONTEXT =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "can-be-security-context");

    /**
     * Returns the value of the {@link #CAN_BE_SECURITY_CONTEXT
     * CAN_BE_SECURITY_CONTEXT} property as defined by this proxy.
     * 
     * @return true if this field can be a security context field
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CAN_BE_SECURITY_CONTEXT CAN_BE_SECURITY_CONTEXT}
     *             property.
     */
    boolean getCanBeSecurityContext() throws WvcmException;

    /** Whether or not this field is a security context field */
    PropertyName<Boolean> IS_SECURITY_CONTEXT =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-security-context");

    /**
     * Returns the value of the {@link #IS_SECURITY_CONTEXT IS_SECURITY_CONTEXT}
     * property as defined by this proxy.
     * 
     * @return True if this field is a security context field
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_SECURITY_CONTEXT IS_SECURITY_CONTEXT} property.
     */
    boolean getIsSecurityContext() throws WvcmException;

    /** The column name for this field in the database table for its record */
    PropertyName<String> DB_NAME =
        new PropertyName<String>(PROPERTY_NAMESPACE, "db-name");

    /**
     * Returns the value of the {@link #DB_NAME DB_NAME} property as defined by
     * this proxy.
     * 
     * @return A String containing the database column name for this field
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DB_NAME DB_NAME} property.
     */
    String getDbName() throws WvcmException;

    /** Whether or not this field is owned by the system (built-in) */
    PropertyName<Boolean> IS_SYSTEM_OWNED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-system-owned");
    
    /**
     * Returns the value of the {@link #IS_SYSTEM_OWNED IS_SYSTEM_OWNED}
     * property as defined by this proxy.
     * 
     * @return True if this field is system owned.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_SYSTEM_OWNED IS_SYSTEM_OWNED} property.
     */
    boolean getIsSystemOwned() throws WvcmException;
    
    /** For a back reference field, this property represent the name of another field
     *  which forward references to this field.
     */
    PropertyName<String> BACK_REFERENCE_FIELDNAME =
        new PropertyName<String>(PROPERTY_NAMESPACE, "back_reference_fieldname");

    
    /**
     * Returns the value of the {@link #BACK_REFERENCE_FIELDNAME}
     * property as defined by this proxy.
     * 
     * @return The name of another field which forward references to this field.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #BACK_REFERENCE_FIELDNAME} property.
     */
    String getBackReferenceFieldName() throws WvcmException;
    
    /**
     * Returns a list of the comparison operators that may be used with this
     * field in a {@link CqQuery.FilterLeaf} construct.
     */
    PropertyName<Operation[]> LEGAL_COMP_OPS =
        new PropertyName<Operation[]>(PROPERTY_NAMESPACE, "legal-comp-ops");

    /**
     * Returns the value of the {@link #LEGAL_COMP_OPS LEGAL_COMP_OPS} property
     * as defined by this proxy.
     * 
     * @return An array of CqQuery.Filter.Operation enumerators, each specifying
     *         a comparison operator that can be used with this field. Will
     *         never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #LEGAL_COMP_OPS LEGAL_COMP_OPS} property.
     */
    Operation[] getLegalCompOps() throws WvcmException;

    /**
     * true if and only if this property may be used in forming queries.
     */
    public static final PropertyName<Boolean> IS_AVAILABLE_FOR_QUERIES =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-available-for-queries");

    /**
     * Returns the value of the
     * {@link CqFieldDefinition#IS_AVAILABLE_FOR_QUERIES
     * IS_AVAILABLE_FOR_QUERIES} property as defined by this proxy.
     * 
     * @return <b>true</b> if this property may be used in a query filter.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqFieldDefinition#IS_AVAILABLE_FOR_QUERIES
     *             IS_AVAILABLE_FOR_QUERIES} property.
     */
    public boolean getIsAvailableForQueries() throws WvcmException;

    /**
     * The data type of the values of the field defined by this field
     * definition using the generic StpProperty.Type enumeration.
     * 
     * @see #FIELD_TYPE
     */
    public static final PropertyName<StpProperty.Type> VALUE_TYPE =
        new PropertyName<StpProperty.Type>(PROPERTY_NAMESPACE, "value-type");

    /**
     * Returns the value of the {@link CqFieldDefinition#VALUE_TYPE} property as
     * defined by this proxy.
     * 
     * @return The value type. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqFieldDefinition#VALUE_TYPE} property.
     *             
     * @see #getFieldType()
     */
    public StpProperty.Type getValueType() throws WvcmException;
}
